@@ -14,7 +14,6 @@ from torchvision import datasets, models, transforms
 import torch.nn.functional as F
 import imageio
 import cv2
-from fht import *
 import matplotlib.pyplot as plt
 import PIL
 
@@ -29,9 +28,11 @@ def optimized_order(input_batch, mask):
         
         img= img.astype('float64')
 
-        img_had = fht2(img);
+        H = walsh_matrix(len(img))
+        img_had = wh.walsh2(img,H)/len(img)
         img_had = np.multiply(img_had, msk);
-        img_out = fht2(img_had);
+        H = walsh_matrix(len(img_had))
+        img_out = wh.walsh2(img_had,H)/len(img_had)
 
         output_batch[i,0,:,:] = torch.from_numpy(img_out);
     return output_batch
@@ -147,9 +148,11 @@ def optimized_order_vid(input_batch, msk):
         img = input_batch[i, 0, :, :];
         img = img.cpu().detach().numpy();
         img= img.astype('float64')
-        img_had = fht2(img);
+        H = walsh_matrix(len(img))
+        img_had = wh.walsh2(img,H)/len(img)
         img_had = np.multiply(img_had, msk);
-        img_out = fht2(img_had);
+        H = walsh_matrix(len(img_had))
+        img_out = wh.walsh2(img_had,H)/len(img_had)
 
         output_batch[i,0,:,:] = torch.from_numpy(img_out);
     output_batch = output_batch.view([batch_size, seq_length, c, h, w])
